/*
 * Copyright (c) 2002, Peter Bentley
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 * 
 *   Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 * 
 *   Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in the
 * documentation and/or other materials provided with the distribution.
 * 
 *   The name Peter Bentley may not be used to endorse or promote
 * products derived from this software without specific prior
 * written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * $Id: omd_disc.h,v 1.9 2002/11/07 09:09:11 pete Exp $
 *
 * ** omd_disc.h - Disc related functions
 */

#define OMD_DISC_CAPACITY_RAW_SIZE	28
#define OMD_TRACK_INFO_RAW_SIZE		6
typedef struct {
    uint8		flags;		/* Eg write protected		*/
    uint8		capacity_info[OMD_DISC_CAPACITY_RAW_SIZE];
    omd_time_t		time_max;	/* Max time disc can store	*/
    omd_time_t		time_used;	/* Time used on disc		*/
    omd_time_t		time_avail;	/* Time left			*/
    uint8		track_info[OMD_TRACK_INFO_RAW_SIZE];
    uint16		track_count;	/* Number of tracks on disc	*/
} omd_disc_info_t;

#define OMD_DISC_FLAG_PROTECTED 0x40	/* Write protect tab is on	*/
#define OMD_DISC_FLAG_RECORDABLE 0x10	/* Media is recordable		*/

#define omd_disc_is_protected( x )  ( ((x)->flags) & OMD_DISC_FLAG_PROTECTED )
#define omd_disc_is_recordable( x ) ( ((x)->flags) & OMD_DISC_FLAG_RECORDABLE )
#define omd_disc_is_writable( x )   ( omd_disc_is_recordable( (x) ) && \
				      ! (omd_disc_is_protected( (x) )))

omd_disc_info_t *omd_disc_get_info( omd_unit_t *unit,
				    omd_disc_info_t *info);
int omd_disc_get_track_count( omd_unit_t *unit, omd_disc_info_t *discinfo );
int omd_disc_set_raw_title( omd_unit_t *unit, const uint8 *title );
