/*
 * Copyright (c) 2002, Peter Bentley
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 * 
 *   Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 * 
 *   Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in the
 * documentation and/or other materials provided with the distribution.
 * 
 *   The name Peter Bentley may not be used to endorse or promote
 * products derived from this software without specific prior
 * written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * $Id: slog.h,v 1.5 2002/11/07 09:09:13 pete Exp $
 *
 * ** slog.h - Simple logging stuff
 */

#define LOG_TRACE	0
#define LOG_DEBUG	1
#define LOG_INFO	2
#define LOG_WARNING	3
#define LOG_ERROR	4
#define LOG_FATAL	5
#define LOG_MAX		5


typedef void (*slog_handler)(int level, const char *msg );

/* Replace handler chain with a single handler */
void *slog_set_handler( int level, slog_handler handler );
/* Add a new handler on the end of a chain */
void *slog_add_handler( int level, slog_handler handler );
/* Set the level of a handler */
void slog_set_handler_level( void *handler, int level );
/* Get the first handler in the chain */
void *slog_get_handler();

/* Actually log something */
void slog( int level, const char *fmt, ... );

/* Hex dump data to the log */
void slog_hex( int level, const char *prefix,
	       const unsigned char *data, int length);


#ifndef CVSID
#ifdef __GNUC__
#define CVSID(id) static char *cvsid() { return cvsid( id ); }
#else
#define CVSID(id) static const char *cvsid = id;
#endif
#endif

